/*
 *
 *   Copyright (c) International Business Machines  Corp., 2012
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * NAME
 *      libuuid_api_test.c
 *
 * DESCRIPTION
 *         test libuuid apis for generation, compare, copy and validation
 * Author
 * 06/11/2012   Sohny Thomas sohthoma@in.ibm.com
 */

/*
 * History
 *              Initial version : sohny thomas <sohthoma@in.ibm.com>
 */

#include<stdio.h>
#include<stdlib.h>
#include<uuid.h>
#include<time.h>
#include<string.h>

uuid_t test_uuid, cmp_uuid;
char *uuid_out;
time_t uuidtime, test_time;

int check_uuid_clear(void)
{
    uuid_clear(test_uuid);
    if (uuid_is_null(test_uuid))
        return 0;

    return -1; /* uuid varible was not null after clear */
}

int check_uuid_generate(void)
{
    uuid_generate(test_uuid);
    /*Above call does not return any value so check
 *         with is_null for successfull generation*/
    if (!uuid_is_null(test_uuid))
        return 0;

    return -1;
}

int check_uuid_gentime(void)
{
    test_time = time(NULL);
    uuid_generate_time(test_uuid);
    /*Above call does not return any value so check
 *         with is_null for successfull generation*/
    if (!uuid_is_null(test_uuid))
        return 0;

    return -1;
}

int check_uuid_gentimesafe(void)
{
    uuid_generate_time_safe(test_uuid);
    if (!uuid_is_null(test_uuid))
        return 0;

    return -1;
}


int check_uuid_unparse(void)
{
    uuid_out = malloc(sizeof(char)*36);
    memset(uuid_out, "", 36);
    uuid_generate(test_uuid);
    uuid_unparse(test_uuid, uuid_out);
    if (strlen(uuid_out) != 0) {
        printf("UUID generated : %s\n", uuid_out);
        return 0;
    }
    return -1;
}

int check_uuidcopy(void)
{
    uuid_generate(test_uuid);
    uuid_copy(cmp_uuid,test_uuid);
    return memcmp(cmp_uuid, test_uuid, sizeof(uuid_t));
}

int check_uuidcmp(void)
{
    int tst_flag;
    tst_flag = 0;
    uuid_generate(test_uuid);
        uuid_copy(cmp_uuid, test_uuid);
    if (uuid_compare(cmp_uuid, test_uuid))
        tst_flag = -1;

    uuid_generate(cmp_uuid); /*check for differing values*/
    if (!uuid_compare(cmp_uuid, test_uuid))
        tst_flag = -1;

    return tst_flag;
}


int check_uuidparse(void)
{
    uuid_out = "4c5b8010-5c9a-4233-a65b-6dd54eb26a8c\0";
    printf("To Parse UUID : %s\n", uuid_out);
    if (uuid_parse(uuid_out, test_uuid))
        return -1;

    return 0;
}

int check_uuidtime(void)
{
    check_uuid_gentime();
    uuidtime = uuid_time(test_uuid, NULL);
    if (test_time!=uuidtime)
        return -1;

    return 0;
}

int usage (char *prgname[])
{

    printf (" usage: %s <<test number>> \n" \
        "   1 : test UUID NULL and clear \n" \
        "   2 : test UUID generate \n" \
        "   3 : test UUID generate with system time \n" \
        "   4 : test UUID generate with system time safe mode \n" \
        "   5 : test UUID unparse \n" \
        "   6 : test UUID copy \n" \
        "   7 : test UUID compare \n" \
        "   8 : test UUID get time generated \n", prgname[0]);

    return -1;
}

int main(int argc, char *argv[])
{
    if (argc==2) {
        switch (*(argv[1])) {
        case '1':
            return check_uuid_clear();
        case '2':
            return check_uuid_generate();
        case '3':
            return check_uuid_gentime();
        case '4':
            return check_uuid_gentimesafe();
        case '5':
            return check_uuid_unparse();
        case '6':
            return check_uuidcopy();
        case '7':
             return check_uuidcmp();
        case '8':
            return check_uuidtime();
        }
    }
    return usage(argv);
}
